package org.rcisoft.common.util.outClient;


//import com.zgiot.zx.schedule.service.INotificationService;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.bmessage.dao.BMessageRepository;
import org.rcisoft.business.bmessage.entity.BMessage;
import org.rcisoft.business.bnotice.entity.BNotice;
import org.rcisoft.business.bnotice.service.BNoticeService;
import org.rcisoft.common.model.InfoTypeEnum;
import org.rcisoft.common.util.feignDto.*;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.result.NotiRet;
import org.rcisoft.core.result.ResultCode;
import org.rcisoft.core.result.Ret;
import org.rcisoft.common.util.feignClient.NotificationFeignClient;
import org.rcisoft.core.util.UrlUtil;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.dept.dao.SysDeptRepository;
import org.rcisoft.sys.user.dao.SysUserMapper;
import org.rcisoft.sys.user.dto.MyInfoDTO;
import org.rcisoft.sys.user.entity.SysUser;
import org.rcisoft.sys.user.service.SysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 消息通知接口调用
 * @author David
 */
@Component
@Slf4j
public class MTNotificationApiRequestClient {

    @Value("${notice.properties.urlCourse}")
    public String noticeUrlCourse;

    @Value("${notice.properties.urlTrain}")
    public String noticeUrlTrain;

    @Value("${notice.properties.urlExam}")
    public String noticeUrlExam;

    @Value("${server.type}")
    private String serverType;

    @Value("${notice.defaultUrl}")
    private String defaultUrl;

    @Value("${appId.noticeAppId}")
    private Long noticeAppId;

    @Autowired
    private NotificationFeignClient notificationFeignClient;
    @Autowired
    private MTCotactApiRequestClient mtCotactApiRequestClient;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    MTMicroappApiRequestClient microappApiRequestClient;

    @Autowired
    BNoticeService bNoticeServiceImpl;


    @Autowired
    BMessageRepository bMessageRepository;
    /**
     * 发送通知
     * @param
     * @return
     */
    public boolean send(BNotice message, String noticeTitle, String type, String urlType) {
        if("dev".equalsIgnoreCase(serverType)) {
            return true;
        }
        NotiRet ret = new NotiRet();

        String noticeType = InfoTypeEnum.EXAN.getLable();
        if (InfoTypeEnum.REPLY_DISCUSS.getValue().equals(message.getInfoType())){
            noticeType = InfoTypeEnum.REPLY_DISCUSS.getLable();
        }
        if (InfoTypeEnum.OFFLINE_TRAIN.getValue().equals(message.getInfoType())){
            noticeType = InfoTypeEnum.OFFLINE_TRAIN.getLable();
        }
        if (InfoTypeEnum.ONLINE_LESSON.getValue().equals(message.getInfoType())){
            noticeType = InfoTypeEnum.ONLINE_LESSON.getLable();
        }

        SysUser senderUser = new SysUser();
        if (StringUtils.isNotEmpty(message.getSenderId())){
            List<String> ids = new ArrayList<>();
            ids.add(message.getSenderId());
            MTUserGetsReqDTO mtUserGetsReqDTO = new MTUserGetsReqDTO();
            mtUserGetsReqDTO.setCorpId(message.getCorpId());
            mtUserGetsReqDTO.setIds(ids);
            List<MTUserInfoRspDTO> mtUserInfoRspDTOList = mtCotactApiRequestClient.userGets(mtUserGetsReqDTO);
            if (mtUserInfoRspDTOList != null && mtUserInfoRspDTOList.size() > 0){
                mtUserInfoRspDTOList.forEach(mtUserInfoRspDTO -> {
                    if (mtUserInfoRspDTO.getId().equals(message.getSenderId())) {
                        senderUser.setName(mtUserInfoRspDTO.getName());
                        senderUser.setHeadPic(mtUserInfoRspDTO.getAvatar());
                        senderUser.setAccountId(mtUserInfoRspDTO.getAccountId());
                    }
                });
            }
        }else {
            senderUser.setName("智学");
            senderUser.setHeadPic(defaultUrl);
        }

        if (null != message.getRecipientIds() && message.getRecipientIds().size() > 0 && message.getRecipientIds().get(0) != null) {

            List<Long> recipientIdLongList = new ArrayList<>();
            message.getRecipientIds().forEach(recipientId -> {
                recipientIdLongList.add(Long.parseLong(recipientId));
            });
            HaveJurisdictionReqDTO haveJurisdictionReqDTO = new HaveJurisdictionReqDTO(Long.parseLong(message.getCorpId()),recipientIdLongList);
            log.debug("----------------------通知-权限验证---------------------");
            Ret haveJurisList =  microappApiRequestClient.haveJurisdictionUserFast(haveJurisdictionReqDTO);
            log.debug("----------------------通知-权限验证返回结果----------------------"+haveJurisList);
            if (haveJurisList.getData() == null){
                return false;
            }
            List<String> haveJurisAccountIds = new ArrayList<>();

            List<String> getUserAccountList = new ArrayList<>();
            List<String> getUserAccountLongList = (List<String>)haveJurisList.getData();
            getUserAccountLongList.forEach(getUserAccount ->{
                getUserAccountList.add(getUserAccount);
            });

            MTUserGetsAccountReqDTO mtUserGetsReqDTO = new MTUserGetsAccountReqDTO(message.getCorpId(),getUserAccountList);
            List<MTAccountIdRspDTO> mtAccountIdRspDTOList = mtCotactApiRequestClient.accountGetUserHead(mtUserGetsReqDTO);
            mtAccountIdRspDTOList.forEach(mtAccountIdRspDTO -> {
                haveJurisAccountIds.add(mtAccountIdRspDTO.getAccountId());
            });
            // 发送智信通知
            MTNotificationSendReqDTO mtNotificationSendReqDTO = MTNotificationSendReqDTO.builder().content(message.getInfoText())
                            .channels("ALL").receiverIds(haveJurisAccountIds).build();

            mtNotificationSendReqDTO.setSenderName("智学");
            mtNotificationSendReqDTO.setChannels("ALL");
            mtNotificationSendReqDTO.setCategoryCodes(Arrays.asList(new String[]{"EDUCATION"}));
            mtNotificationSendReqDTO.setSenderId(senderUser.getAccountId());
            mtNotificationSendReqDTO.setSenderName(senderUser.getName());
            mtNotificationSendReqDTO.setCorpId(message.getCorpId());
            mtNotificationSendReqDTO.setTitle(senderUser.getName());

            SimpleDateFormat df = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
            String noticePropertiesUrl = "";
            if ("LESSON".equals(urlType)){
                noticePropertiesUrl = this.noticeUrlCourse + "id=" + message.getLessonId()+"&innerShare=1";
            }else if ("TRAIN".equals(urlType)){
                noticePropertiesUrl = this.noticeUrlTrain + "id=" + message.getLessonId()+"&fromName=Home&role=people"+"&innerShare=1";
            }else if ("EXAM".equals(urlType)){
                noticePropertiesUrl = this.noticeUrlExam;
            }
            if("REPLY_DISCUSS".equals(noticeTitle))
                noticePropertiesUrl = noticePropertiesUrl+"&notice=2";

            String url = noticePropertiesUrl + "&from=zx";

            final String finalNoticeType = noticeType;
            mtNotificationSendReqDTO.setProperties(new HashMap<String, String>() {{
                put("url", url);
                put("createUrl",senderUser.getHeadPic());
                put("createName",senderUser.getName());
                put("area",finalNoticeType);
            }});

            String json = JSON.toJSONString(mtNotificationSendReqDTO);
            log.debug(json);

            try {
                ret = notificationFeignClient.sendMessage(mtNotificationSendReqDTO, "app", noticeAppId);
            } catch (Exception ex) {
                log.error("----------------------------------------------------调用智信通知失败！----------------------------------------------------------"+ex);
                return false;
            }

        }


        if(ret != null && "M0000".equals(ret.getCode())){
            return true;
        }
        return false;
    }

    private String dealWithTitle(String title){
        String result = "";
        switch(title){
            case "评论" :
                result = "REPLY_DISCUSS";
                break;
            default : //可选
        }
        return result;
    }

}
